use nostr::{key::FromPkStr, Event, Filter, Kind};

fn ids_match(filter: &Filter, event: &Event) -> bool {
    if filter.ids.is_empty() {
        log::info!("[FILTER][ids_match] skipped");
        return true;
    }

    log::info!(
        "[FILTER][ids_match] matched: {:?}",
        filter.ids.iter().any(|id| id == &event.id)
    );

    filter.ids.iter().any(|id| id == &event.id)
}

fn kind_match(filter: &Filter, kind: Kind) -> bool {
    if filter.kinds.is_empty() {
        log::debug!("[FILTER][kind_match] skipped");
        return true;
    }

    log::debug!(
        "[FILTER][kind_match] matched: {:?}",
        filter.kinds.iter().any(|k| k == &kind)
    );

    filter.kinds.iter().any(|k| k == &kind)
}

fn pubkeys_match(filter: &Filter, event: &Event) -> bool {
    log::debug!(
        "[FILTER][pubkeys_match] matched: {:?}",
        if let Some((p_tag, p_set)) = filter.generic_tags.get_key_value(&nostr::Alphabet::P) {
            if p_set.is_empty() {
                log::debug!("[FILTER][PUBKEYS] skipped");
                return true;
            }

            return p_set.iter().any(|pk| match pk {
                nostr::GenericTagValue::Pubkey(pk) => pk == &event.pubkey,
                _ => false,
            });
        }
    );
    if let Some((p_tag, p_set)) = filter.generic_tags.get_key_value(&nostr::Alphabet::P) {
        if p_set.is_empty() {
            log::debug!("[FILTER][PUBKEYS] skipped");
            return true;
        }

        return p_set.iter().any(|pk| match pk {
            nostr::GenericTagValue::Pubkey(pk) => pk == &event.pubkey,
            _ => false,
        });
    }

    false
}

fn authors_match(filter: &Filter, event: &Event) -> bool {
    if filter.authors.is_empty() {
        log::debug!("[FILTER][authors_match] skipped");
        return true;
    }

    log::debug!(
        "[FILTER][authors_match] matched: {:?}",
        filter.authors.iter().any(|author| author == &event.pubkey)
    );
    filter.authors.iter().any(|author| author == &event.pubkey)
}

fn delegated_authors_match(filter: &Filter, event: &Event) -> bool {
    log::debug!(
        "[FILTER][delegated_authors_match] matched: {:?}",
        event.tags.iter().any(|tag| {
            if tag.kind() == nostr::TagKind::Delegation {
                let tag = tag.as_vec();
                if let Ok(event_pubkey) = nostr::Keys::from_pk_str(&tag[1]) {
                    let pk = event_pubkey.public_key();
                    return filter.authors.iter().any(|author| author == &pk);
                }
                return false;
            }
            false
        })
    );

    event.tags.iter().any(|tag| {
        if tag.kind() == nostr::TagKind::Delegation {
            let tag = tag.as_vec();
            if let Ok(event_pubkey) = nostr::Keys::from_pk_str(&tag[1]) {
                let pk = event_pubkey.public_key();
                return filter.authors.iter().any(|author| author == &pk);
            }
            return false;
        }
        false
    })
}

fn tag_match(filter: &Filter, event: &Event) -> bool {
    if filter.generic_tags.is_empty() && event.tags.is_empty() {
        return true;
    }
    log::debug!(
        "[FILTER][tag_match] matched: {:?}",
        filter
            .generic_tags
            .iter()
            .any(|(filter_tag_key, filter_tag_value)| {
                event.tags.iter().any(|event_tag| {
                    let event_tag = event_tag.as_vec();
                    let event_tag_key = event_tag[0].clone();
                    let event_tag_value = event_tag[1].clone();

                    if filter_tag_key.to_string() == event_tag_key {
                        return filter_tag_value
                            .iter()
                            .any(|f_tag_val| f_tag_val.to_string() == event_tag_value);
                    };

                    false
                })
            })
    );

    filter
        .generic_tags
        .iter()
        .any(|(filter_tag_key, filter_tag_value)| {
            event.tags.iter().any(|event_tag| {
                let event_tag = event_tag.as_vec();
                let event_tag_key = event_tag[0].clone();
                let event_tag_value = event_tag[1].clone();

                if filter_tag_key.to_string() == event_tag_key {
                    return filter_tag_value
                        .iter()
                        .any(|f_tag_val| f_tag_val.to_string() == event_tag_value);
                };

                false
            })
        })
}

pub fn interested_in_event(filter: &Filter, event: &Event) -> bool {
    ids_match(filter, event)
        && filter.since.map_or(
            {
                log::info!("[FILTER][SINCE][default] matched: {:?}", true);
                true
            },
            |t| {
                log::info!("[FILTER][SINCE] matched: {:?}", event.created_at >= t);
                event.created_at >= t
            },
        )
        && filter.until.map_or(
            {
                log::info!("[FILTER][UNTIL][default] matched: {:?}", true);
                true
            },
            |t| {
                log::info!("[FILTER][UNTIL] matched: {:?}", event.created_at <= t);
                event.created_at <= t
            },
        )
        && kind_match(filter, event.kind)
        && (pubkeys_match(filter, event)
            || authors_match(filter, event)
            || delegated_authors_match(filter, event))
        && tag_match(filter, event)
}

#[cfg(test)]
mod tests {
    use crate::utils::nostr_filter_helpers::interested_in_event;

    #[test]
    fn check_simple_match() {
        let my_keys = nostr::Keys::generate();
        let event = nostr::EventBuilder::text_note("hello", [])
            .to_event(&my_keys)
            .unwrap();

        let k = nostr::Kind::TextNote;
        let filter = nostr::Filter::new()
            .kinds(vec![k])
            .authors(vec![event.pubkey]);

        let res = interested_in_event(&filter, &event);

        dbg!(&res);

        assert!(res);
    }
}
